using System;
using System.Collections;
using System.Text;
using System.Data.OleDb;
using DBSync.DataObject;

namespace DBSync.DataSource
{
    /// <summary>
    /// Class that helps read and write product information to SQL Server
    /// datasource.
    /// </summary>
    internal class OtherDatasource
    {
        private OleDbConnection _connection;
        private string _connString = @"Data Source=localhost; User ID = system; password =;";

        /// <summary>
        /// Returns the connection string of this datasource.
        /// </summary>
        public string ConnString { get { return _connString; } }

        /// <summary>
        /// Establish connection with the datasource.
        /// </summary>
        /// <param name="connString"></param>
        public void Connect(string connString)
        {
            if (connString != "")
                _connString = connString;
            _connection = new OleDbConnection(_connString);
            try
            {
                _connection.Open();
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
                throw ex;
            }
        }

        /// <summary>
        /// Releases the connection.
        /// </summary>
        public void DisConnect()
        {
            try
            {
                if (_connection != null && _connection.State != System.Data.ConnectionState.Closed)
                    _connection.Close();
            }
            catch { }
        }

        /// <summary>
        /// Loads a <see cref="Product"/> object from the datasource.
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public object LoadProduct(Object key)
        {
            object[] productId = { key };
            OleDbDataReader reader = null;
            try
            {
                OleDbCommand cmd = _connection.CreateCommand();
                cmd.CommandText = String.Format(System.Globalization.CultureInfo.InvariantCulture,
                                    "Select ProductID, ProductName, UnitPrice " +
                                    "From Products where ProductID = '{0}'",
                                    productId);
                reader = cmd.ExecuteReader();
                if (reader.Read())
                {
                    Product objProduct = new Product();

                    objProduct.ProductID = Convert.ToInt32(reader["ProductID"]);
                    objProduct.ProductName = reader["ProductName"].ToString();
                    objProduct.UnitPrice = Convert.ToInt32(reader["UnitPrice"]);
                    reader.Close();
                    return objProduct;
                }
                reader.Close();
            }
            catch (Exception e)
            {
                System.Diagnostics.Trace.WriteLine(e);
            }

            return null;
        }
        /// <summary>
        /// returns the top <paramref name="numOfRows"/> from the datasource
        /// </summary>
        /// <param name="number of rows"></param>
        /// <returns></returns>
        public Product[] LoadProducts(int numOfRows)
        {
            if (numOfRows <= 0)
            {
                throw new Exception("invalid parameter value: numOfRows can not be less than 1");
            }

            OleDbDataReader reader = null;
            try
            {
                OleDbCommand cmd = _connection.CreateCommand();
                cmd.CommandText = String.Format(System.Globalization.CultureInfo.InvariantCulture,
                                    "Select ProductID, ProductName, UnitPrice " +
                                    "From Products " + 
                                    "Where ROWNUM <= {0}",
                                    numOfRows);
                reader = cmd.ExecuteReader();

                Product[] objProducts = new Product[numOfRows];
                for (int i = 0; i < numOfRows; i++)
                {
                    if (reader.Read())
                    {
                        objProducts[i] = new Product();

                        objProducts[i].ProductID = Convert.ToInt32(reader["ProductID"]);
                        objProducts[i].ProductName = reader["ProductName"].ToString();
                        objProducts[i].UnitPrice = Convert.ToInt32(reader["UnitPrice"]);
                    }
                }
                reader.Close();
                return objProducts;
            }
            catch (Exception e)
            {
                System.Diagnostics.Trace.WriteLine(e);
            }

            return null;
        }
    }
}
